(document:surround "/std/frame")

;;; Helpers for checklistbox

(define *prev-current-rows* (make-cell #f))

(define (checklistbox-current)
  (let* ((cr (packages current-rows))
	 (pcr (cell-ref *prev-current-rows*)))
    (cond-car (if pcr (lset-difference = cr pcr) cr))))

(define (checklistbox-update-current)
  (cell-set! *prev-current-rows* (packages current-rows)))

(define (checklistbox-revert-current)
  (packages current-rows (cell-ref *prev-current-rows*)))

(define (checklistbox-state num)
  (list-ref (packages state-rows) num))

;;; Features: list of packages

(define *pkg-lists* (make-cell '()))

(define (make-pkg-lists pkg-groups)
  (cell-set! *pkg-lists* (map (lambda(x) (woo-get-option x 'lists)) pkg-groups)))

(define (pkg-install-list)
  (string-join
    (filter-map (lambda (x1 x2) (and x1 x2))
		(packages state-rows)
		(cell-ref *pkg-lists*))
    " "))

;;; Features: Lists

(define (label+required x)
  (cons (woo-get-option x 'label) (woo-get-option x 'required)))


;;; Features: Conflicts
(define *pkg-conflicts* (make-cell '()))

(define (name+conflicts x)
  (cons (woo-get-option x 'name)
	(woo-get-option x 'conflicts)))

(define (get-index lst)
  (lambda(x)
    (list-index (lambda (y) (string=? x y)) lst)))

(define (make-pkg-conflicts pkg-groups)
  (let* ((lst (map name+conflicts pkg-groups))
	 (names (map car lst))
	 (conflicts (map cdr lst)))
    (cell-set! *pkg-conflicts* (map (get-index names) conflicts))))

(define (pkg-conflicts-num num)
  (list-ref (cell-ref *pkg-conflicts*) num))

(define (pkg-conflicts-name num)
  (car (list-ref (packages checklist-rows) num)))

(define (check-pkg-conflicts num)
  (let* ((c-num (pkg-conflicts-num num)))
    (not (and (number? c-num)
	      (checklistbox-state c-num)
	      (begin (document:popup-critical
		       (format #f (_ "Has conflict with ~S") (pkg-conflicts-name c-num)))
		     #t)))))

;;; Features: Disk usage

(define (string-prefix-at-pos str prefix pos)
  (and (string-prefix? prefix str)
       (list-ref (string-cut str #\:) pos)))

(define (pkg-size-label msg)
  (size text (format #f "~A:~A" (_ "Required disk space") msg)))

(define (pkg-message-handler msg)
  (and-let* ((msg (cond-plistq 'message msg #f)))
	    (cond
	      ((string=? msg "pkg-size:start")
	       (pkg-size-label (_ "calculating...")))
	      ((string-prefix-at-pos msg "apt-get:status:disk-size:" 3) =>
		(lambda(result)
		  (pkg-size-label (string-append result "B"))))
	      ((string-prefix-at-pos msg "pkg-size:finish" 2) =>
	        (lambda(result)
		  (and (string=? result "100") (pkg-size-label (_ "unable to calculate"))))))))


(define (pkg-error-handler msg)
  (and-let* ((error-text (cond-plistq 'error msg #f)))
	    (format #t "error-text:~S~%" error-text)))

(define (make-pkg-size)
  (add-mailbox pkg-message-handler)
  (add-mailbox pkg-error-handler)
  (pkg-size-label "0kB"))

(define (remove-pkg-size)
  (remove-mailbox pkg-message-handler)
  (remove-mailbox pkg-error-handler)
  (woo-write "/pkg-size" 'lists ""))

(define (check-pkg-size)
  (woo-write "/pkg-size" 'lists (pkg-install-list)))


;;; UI

(gridbox
  columns "10;80;10"

  ;;
  (spacer)
  (label text (_ "Select packages you want to install:"))
  (spacer)
  ;;
  (spacer)
  (document:id packages (checklistbox
			  (when selected
			    (let ((num (checklistbox-current)))
			      (cond
				(num (if (check-pkg-conflicts num)
				       (begin (checklistbox-update-current) (check-pkg-size))
				       (checklistbox-revert-current)))
				(else (begin (checklistbox-update-current) (check-pkg-size))))))))
  (spacer)
  ;;
  (spacer)
  (document:id size (label text (_ "Preparing...")))
  (spacer))

;;; Logic

(frame:on-next (thunk (remove-pkg-size)
		      (or (null? (cell-ref *pkg-lists*))
			  (begin (frame:replace "/pkg/install" 'lists (pkg-install-list))
				 'cancel))))

(frame:on-back (thunk (remove-pkg-size)))

(document:root
  (when loaded
    (woo-catch/message
      (thunk
	(let* ((pkg-groups (woo-list "/pkg-groups")))
	  (if (null? pkg-groups)
	    (frame:skip)
	    (begin
	      (splash-message (_ "Please wait..."))
	      (document:release)

	      ;;update cache
	      (woo-write "/pkg-update")

	      (make-pkg-lists pkg-groups)
	      (make-pkg-conflicts pkg-groups)
	      (make-pkg-size)
	      (packages checklist-rows (map label+required pkg-groups))
	      (check-pkg-size)

	      (splash-message ""))))))))
